import os
import sqlite3
from pathlib import Path
from sqlite3 import Connection

from otlmow_modelbuilder.GeometrieType import GeometrieType


class GeometrieInMemoryCreator:
    def __init__(self, path: Path):
        self.connection: Connection
        self.path: Path = path
        if path is None or path == '':
            self.file_exists = False
        else:
            self.path = path.resolve()
            self.file_exists = os.path.isfile(self.path)
            if not self.file_exists:
                raise FileNotFoundError(str(self.path) + " is not a valid path. File does not exist.")

    def __enter__(self):
        self.connection = sqlite3.connect(self.path)
        return self

    def __exit__(self, exc_type, exc_value, exc_traceback):
        self.connection.close()

    def perform_read_query(self, query: str, params=None) -> [tuple]:
        if params is None:
            params = {}

        cur = self.connection.cursor()
        return [row for row in cur.execute(query, params)]

    def get_all_geometrie_types(self) -> [GeometrieType]:
        data = self.perform_read_query(query='SELECT * FROM GeometrieType')
        return [GeometrieType(row[0].replace('-test', ''), row[1], row[2], row[3], row[4], row[5], row[6])
                for row in data]





import dataclasses


@dataclasses.dataclass
class OSLODatatypePrimitiveAttribuut:
    name: str = ''
    label: str = ''
    definition: str = ''
    class_uri: str = ''
    kardinaliteit_min: str = ''
    kardinaliteit_max: str = ''
    objectUri: str = ''
    type: str = ''
    overerving: int = -1
    constraints: str = ''
    readonly: int = -1
    usagenote: str = ''
    deprecated_version: str = ''







import logging
import os
import re
from os.path import abspath
from pathlib import Path
from typing import Dict
from urllib.request import urlretrieve
from zipfile import ZipFile

import rdflib
from rdflib import URIRef, Graph, RDF

from otlmow_modelbuilder.AbstractDatatypeCreator import AbstractDatatypeCreator
from otlmow_modelbuilder.GenericBuilderFunctions import get_white_space_equivalent
from otlmow_modelbuilder.OSLOCollector import OSLOCollector
from otlmow_modelbuilder.SQLDataClasses.OSLOEnumeration import OSLOEnumeration
from otlmow_modelbuilder.HelperFunctions import wrap_in_quotes


class KeuzelijstWaarde:
    def __init__(self, invulwaarde='', label='', definitie='', objectUri='', status=''):
        self.invulwaarde = invulwaarde
        self.label = label
        self.definitie = definitie
        self.objectUri = objectUri
        self.status = status

    def print(self):
        if self.status in ('', 'ingebruik'):
            return self.invulwaarde

        return f'{self.invulwaarde} ({self.status})'


class OTLEnumerationCreator(AbstractDatatypeCreator):
    default_environment = 'prd'
    graph_dict: Dict[str, Dict[str, Graph]] = {'prd': {}, 'tei': {}, 'dev': {}}
    oslo_github_branch_mapping = {
        'prd': 'master',
        'tei': 'test',
        'dev': 'dev'
    }

    def __init__(self, oslo_collector: OSLOCollector, env: str = default_environment, 
                 include_kl_test_keuzelijst: bool = False):
        super().__init__(oslo_collector)
        self.oslo_collector = oslo_collector
        self.env = env
        self.include_kl_test_keuzelijst = include_kl_test_keuzelijst
        logging.info("Created an instance of OTLEnumerationCreator")

    def __enter__(self):
        self.path_zip_file = Path(__file__).parent / "all.ttl.zip"
        self.path_ttl_file = Path(__file__).parent / "all.ttl"
        if self.env != 'unittest':
            self.graph_dict[self.env] = self.download_unzip_and_parse_to_dict(env=self.env)
            logging.info("Downloaded, unzipped and parsed the enumerations ttl file")
        if self.include_kl_test_keuzelijst:
            self.add_kl_test_keuzelijst(env=self.env)
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.path_zip_file.exists():
            self.path_zip_file.unlink()
        if self.path_ttl_file.exists():
            self.path_ttl_file.unlink()

    def add_kl_test_keuzelijst(self, env):
        kl_test_keuzelijst_path = Path(__file__ ).parent.parent / 'UnitTests/KlTestKeuzelijst.ttl'
        if not kl_test_keuzelijst_path.exists():
            url = 'https://raw.githubusercontent.com/davidvlaminck/OTLMOW-ModelBuilder/refs/heads/master/UnitTests/KlTestKeuzelijst.ttl'
            urlretrieve(url, kl_test_keuzelijst_path)
        self.graph_dict[env].update(self.parse_graph_to_dict(path_ttl_file=kl_test_keuzelijst_path))

    def create_block_to_write_from_enumerations(self, oslo_enumeration: OSLOEnumeration, enumeration_validation_rules,
                                                environment: str = default_environment) -> [str]:
        if not isinstance(oslo_enumeration, OSLOEnumeration):
            raise ValueError("Input is not a OSLOEnumeration")

        if oslo_enumeration.objectUri == '':
            raise ValueError(f"OSLOEnumeration.objectUri is invalid. Value = '{oslo_enumeration.objectUri}'")

        if oslo_enumeration.objectUri not in enumeration_validation_rules['valid_uri_and_types'].keys():
            match_re = False
            for regex in enumeration_validation_rules["valid_regexes"]:
                match_re = re.match(pattern=regex, string=oslo_enumeration.objectUri)
                if match_re:
                    break
            if not match_re:
                raise ValueError(
                    f"OSLOEnumeration.objectUri is invalid. Value = '{oslo_enumeration.objectUri}'")

        if oslo_enumeration.name == '':
            raise ValueError(f"OSLOEnumeration.name is invalid. Value = '{oslo_enumeration.name}'")

        return self.create_block_to_write_from_enumeration(oslo_enumeration=oslo_enumeration, environment=environment)

    def create_block_to_write_from_enumeration(self, oslo_enumeration: OSLOEnumeration,
                                               environment: str = default_environment) -> [str]:
        keuzelijst_waardes = self.get_keuzelijstwaardes_by_uri(oslo_enumeration.codelist, env=environment)
        adm_status = self.get_adm_status_by_uri(oslo_enumeration.codelist, env=environment)

        datablock = ['# coding=utf-8',
                     'from otlmow_model.OtlmowModel.BaseClasses.KeuzelijstField import KeuzelijstField']
        if len(keuzelijst_waardes) > 0:
            datablock.append('from otlmow_model.OtlmowModel.BaseClasses.KeuzelijstWaarde import KeuzelijstWaarde')

        datablock.extend(['',
                          '',
                          f'# Generated with {self.__class__.__name__}. To modify: extend, do not edit',
                          f'class {oslo_enumeration.name}(KeuzelijstField):',
                          f'    """{oslo_enumeration.definition}"""',
                          f'    naam = {wrap_in_quotes(oslo_enumeration.name)}',
                          f'    label = {wrap_in_quotes(oslo_enumeration.label)}',
                          f'    objectUri = {wrap_in_quotes(oslo_enumeration.objectUri)}',
                          f'    definition = {wrap_in_quotes(oslo_enumeration.definition)}'])

        if adm_status is not None and adm_status != '':
            datablock.append(f'    status = {wrap_in_quotes(adm_status)}')
        if oslo_enumeration.deprecated_version != '':
            oslo_enumeration.deprecated_version = oslo_enumeration.deprecated_version.split('-')[0]
            datablock.append(f'    deprecated_version = {wrap_in_quotes(oslo_enumeration.deprecated_version)}')
        datablock.append(f'    codelist = {wrap_in_quotes(oslo_enumeration.codelist)}')
        datablock.append('    options = {')

        for waarde in sorted(keuzelijst_waardes, key=lambda w: w.invulwaarde):
            if oslo_enumeration.objectUri == 'https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#KlFiguratieCode':
                waarde.definitie = waarde.definitie.replace('²', '^2')
                waarde.invulwaarde = waarde.invulwaarde.replace("'", "\\'")
                waarde.label = waarde.label.replace("'", "\\'")
                waarde.objectUri = waarde.objectUri.replace("'", "\\'")

            whitespace = get_white_space_equivalent(f"        '{waarde.invulwaarde}': KeuzelijstWaarde(")
            datablock.append(f"        '{waarde.invulwaarde}': KeuzelijstWaarde(invulwaarde='{waarde.invulwaarde}',")
            datablock.append(f"{whitespace}label={wrap_in_quotes(waarde.label)},")
            if waarde.status != '':
                datablock.append(f"{whitespace}status='{waarde.status}',")
            if waarde.definitie != '':
                datablock.append(f"{whitespace}definitie={wrap_in_quotes(waarde.definitie)},")
            datablock.append(f"{whitespace}objectUri='{waarde.objectUri}'),")

        if len(keuzelijst_waardes) > 0:
            datablock[-1] = datablock[-1][:-1]
        datablock.append('    }')

        # dummy values part
        datablock.append("")
        datablock.append("    @classmethod")
        datablock.append("    def create_dummy_data(cls):")
        datablock.append("        return cls.create_dummy_data_keuzelijst(cls.options)")
        datablock.append('')

        return datablock

    @classmethod
    def get_graph(cls, keuzelijstnaam: str, env: str = default_environment):
        keuzelijst_graph = OTLEnumerationCreator.graph_dict[env].get(keuzelijstnaam)
        if keuzelijst_graph is not None:
            return keuzelijst_graph
        raise ValueError(f"Graph for {keuzelijstnaam} not found in the graph_dict")

    def download_unzip_and_parse_to_dict(self, env: str = default_environment) -> Dict[str, Graph]:
        directory_to_extract_to = self.path_zip_file.parent
        urlretrieve(f"https://github.com/Informatievlaanderen/OSLO-codelistgenerated/raw/refs/heads/wegenenverkeer-{self.oslo_github_branch_mapping[env]}/all.ttl.zip", self.path_zip_file)
        with ZipFile(self.path_zip_file, 'r') as zip_ref:
            zip_ref.extractall(directory_to_extract_to)

        return self.parse_graph_to_dict(path_ttl_file=self.path_ttl_file)

    @staticmethod
    def parse_graph_to_dict(path_ttl_file: Path) -> Dict[str, Graph]:
        g = rdflib.Graph()
        g.parse(path_ttl_file, format="turtle")

        keuzelijst_dict = {}
        keuzelijst_uris = set(g.subjects(predicate=RDF.type, object=URIRef('http://www.w3.org/2004/02/skos/core#ConceptScheme')))

        for keuzelijst_uri in keuzelijst_uris:
            keuzelijst_graph = Graph()
            for triple in g.triples((keuzelijst_uri, None, None)):
                keuzelijst_graph.add(triple)

            keuzelijst_waarde_uris = g.subjects(predicate=URIRef('http://www.w3.org/2004/02/skos/core#inScheme'),
                                                object=keuzelijst_uri)
            for keuzelijst_waarde_uri in keuzelijst_waarde_uris:
                for triple in g.triples((keuzelijst_waarde_uri, None, None)):
                    keuzelijst_graph.add(triple)

            keuzelijst_dict[str(keuzelijst_uri)] = keuzelijst_graph
        rdflib.term.URIRef('https://wegenenverkeer.data.vlaanderen.be/id/conceptscheme/KlTestKeuzelijst')
        return keuzelijst_dict

    @classmethod
    def get_keuzelijstwaardes_by_uri(cls, uri: str, env: str = default_environment) -> [KeuzelijstWaarde]:
        g = cls.graph_dict[env][uri]
        return cls.get_keuzelijstwaardes_from_graph(g, env)

    @classmethod
    def get_keuzelijstwaardes_from_graph(cls, g: Graph, env: str = default_environment):
        lijst_keuze_opties = []

        subjects = set(g.subjects(predicate=None, object=None))
        distinct_subjects_list = sorted(subjects, key=lambda x: str(x))

        for distinct_subject in distinct_subjects_list:
            subject_str = str(distinct_subject)
            if subject_str.startswith('https://wegenenverkeer.data.vlaanderen.be/id/conceptscheme/'):
                continue
            elif env == 'tei' and subject_str.startswith(
                    'https://wegenenverkeer-test.data.vlaanderen.be/id/conceptscheme/'):
                continue
            waarde = KeuzelijstWaarde()
            waarde.objectUri = subject_str
            status = g.value(subject=distinct_subject, predicate=URIRef('https://www.w3.org/ns/adms#status'))
            if status is not None:
                waarde.status = str(status).replace(
                    'https://wegenenverkeer.data.vlaanderen.be/id/concept/KlAdmsStatus/', '')
            waarde.invulwaarde = str(
                g.value(subject=distinct_subject, predicate=URIRef('http://www.w3.org/2004/02/skos/core#notation')))
            waarde.definitie = str(
                g.value(subject=distinct_subject, predicate=URIRef('http://www.w3.org/2004/02/skos/core#definition')))
            waarde.label = str(
                g.value(subject=distinct_subject, predicate=URIRef('http://www.w3.org/2004/02/skos/core#prefLabel')))
            lijst_keuze_opties.append(waarde)

        return lijst_keuze_opties

    @classmethod
    def get_adm_status_by_uri(cls, keuzelijst_uri: str, env: str = default_environment) -> str:
        g = OTLEnumerationCreator.get_graph(keuzelijst_uri, env=env)
        return cls.get_adm_status_from_graph(g, uri=keuzelijst_uri, env=env)

    @classmethod
    def get_adm_status_from_graph(cls, g: Graph, uri: str, env: str = default_environment) -> str:
        if env == 'tei':
            uri = uri.replace('wegenenverkeer', 'wegenenverkeer-test')

        status = g.value(subject=URIRef(uri), predicate=URIRef('https://www.w3.org/ns/adms#status'))

        if status is not None:
            return str(status).replace('https://wegenenverkeer.data.vlaanderen.be/id/concept/KlAdmsStatus/', '')
        else:
            return ''



import json
import os
from pathlib import Path
from typing import Dict


def _check_settings_with_default(settings_path: Path) -> Path:
    if settings_path is None:
        current_file_path = Path(__file__)
        settings_path = Path(current_file_path.parent / 'settings_otlmow_modelbuilder.json')

    if not os.path.isfile(settings_path):
        raise FileNotFoundError(f'{settings_path} is not a valid path. File does not exist.')

    return settings_path


def load_settings(settings_path: Path) -> Dict:
    settings_path = _check_settings_with_default(settings_path=settings_path)

    try:
        with open(settings_path) as settings_file:
            return json.load(settings_file)
    except OSError:
        raise ImportError(f'Could not open the settings file at {settings_path}')



from pathlib import Path
from typing import Type, Union

from otlmow_modelbuilder.GeometrieInMemoryCreator import GeometrieInMemoryCreator
from otlmow_modelbuilder.GeometrieType import GeometrieType


class GeometrieArtefactCollector:
    def __init__(self, path: Path):
        self.path = path

        self.geometrie_types: [GeometrieType] = None

    def collect_all(self):
        with GeometrieInMemoryCreator(self.path) as memory_creator:
            self.geometrie_types = memory_creator.get_all_geometrie_types()

    def find_by_objectUri(self, objectUri: str) -> Union[Type[GeometrieType], None]:
        result = list(filter(lambda g: g.objectUri == objectUri, self.geometrie_types))
        if len(result) == 0:
            return None
        return result[0]




import dataclasses


@dataclasses.dataclass
class OSLORelatie:
    bron_overerving: str
    doel_overerving: str
    bron_uri: str
    doel_uri: str
    objectUri: str
    richting: str
    usagenote: str
    deprecated_version: str



import logging
import re
from typing import Dict

from otlmow_modelbuilder.AbstractDatatypeCreator import AbstractDatatypeCreator
from otlmow_modelbuilder.OSLOCollector import OSLOCollector
from otlmow_modelbuilder.SQLDataClasses.OSLODatatypeComplex import OSLODatatypeComplex


class OTLComplexDatatypeCreator(AbstractDatatypeCreator):
    def __init__(self, oslo_collector: OSLOCollector):
        super().__init__(oslo_collector)
        logging.info("Created an instance of OTLComplexDatatypeCreator")

    def create_block_to_write_from_complex_types(self, oslo_datatype_complex: OSLODatatypeComplex,
                                                 complex_datatype_validation_rules: Dict, model_location='') -> [str]:
        if not isinstance(oslo_datatype_complex, OSLODatatypeComplex):
            raise ValueError(f"Input is not a OSLODatatypeComplex")

        if oslo_datatype_complex.objectUri == '':
            raise ValueError(f"OSLODatatypeComplex.objectUri is invalid. Value = '{oslo_datatype_complex.objectUri}'")

        if oslo_datatype_complex.objectUri in complex_datatype_validation_rules['valid_uri_and_types'].keys():
            pass
        else:
            match_re = False
            for regex in complex_datatype_validation_rules["valid_regexes"]:
                match_re = re.match(pattern=regex, string=oslo_datatype_complex.objectUri)
                if match_re:
                    break
            if not match_re:
                raise ValueError(
                    f"OSLODatatypeComplex.objectUri is invalid. Value = '{oslo_datatype_complex.objectUri}'")

        if oslo_datatype_complex.name == '':
            raise ValueError(f"OSLODatatypeComplex.name is invalid. Value = '{oslo_datatype_complex.name}'")

        return self.create_block_to_write_from_complex_primitive_or_union_types(
            oslo_datatype_complex, type_field='Complex', model_location=model_location,
            valid_uri_and_types=complex_datatype_validation_rules['valid_uri_and_types'])

    @staticmethod
    def get_unit_from_constraints(constraints: str) -> str:
        if constraints == '':
            raise ValueError
        split_text = constraints.split('"')
        return split_text[1]



import cProfile
import logging
from pathlib import Path

from otlmow_modelbuilder.ModelBuilder import ModelBuilder


def create_model():
    logging.basicConfig(level=logging.INFO, format='%(message)s')

    current_dir = Path(__file__).parent
    otl_subset_path = Path(current_dir / 'InputFiles' / 'OTL 2.12.db')
    GA_file_path = Path(current_dir / 'InputFiles' / 'Geometrie_Artefact_2.12.db')
    model_directory = Path(current_dir.parent.parent / 'OTLMOW-Model/otlmow_model')
    settings_path = Path(current_dir / 'settings_otlmow_modelbuilder.json')

    ModelBuilder.build_otl_datamodel(otl_subset_location=otl_subset_path, geometry_artefact_location=GA_file_path,
                                     directory=model_directory, settings_path=settings_path,
                                     environment='prd')


if __name__ == '__main__':
    #cProfile.run('create_model()')
    create_model()



import dataclasses


@dataclasses.dataclass
class GeometrieType:
    objectUri: str = ''
    label_nl: str = ''
    geen_geometrie: int = -1
    punt3D: int = -1
    lijn3D: int = -1
    polygoon3D: int = -1
    gewijzigd_sinds: str = ''



from pathlib import Path

from otlmow_modelbuilder.GeometrieArtefactCollector import GeometrieArtefactCollector
from otlmow_modelbuilder.OSLOCollector import OSLOCollector
from otlmow_modelbuilder.OTLModelCreator import OTLModelCreator
from otlmow_modelbuilder.SettingsManager import load_settings


class ModelBuilder:
    @staticmethod
    def build_otl_datamodel(otl_subset_location: Path,
                            directory: Path,
                            environment: str = 'prd',
                            geometry_artefact_location: Path = None,
                            settings_path: Path = None,
                            include_kl_test_keuzelijst: bool = False):
        if directory is None:
            this_file_path = Path(__file__)
            directory = this_file_path.parent

        subset_collector = OSLOCollector(otl_subset_location)
        subset_collector.collect_all()

        geo_artefact_collector = None
        if geometry_artefact_location is not None:
            geo_artefact_collector = GeometrieArtefactCollector(geometry_artefact_location)

        if geo_artefact_collector is not None:
            geo_artefact_collector.collect_all()

        settings = load_settings(settings_path)

        OTLModelCreator.create_full_model(
            directory=directory, environment=environment, settings=settings, oslo_collector=subset_collector,
            geo_artefact_collector=geo_artefact_collector, include_kl_test_keuzelijst=include_kl_test_keuzelijst)



from typing import Dict


def get_single_field_from_type_uri(field_type: str):
    if field_type.startswith('https://wegenenverkeer.data.vlaanderen.be/ns/') and '#' in field_type:
        return field_type.split('#')[1]

    if field_type is None:
        return ''
    elif field_type == 'http://www.w3.org/2001/XMLSchema#decimal':
        return 'FloatOrDecimalField'
    elif field_type == 'http://www.w3.org/2001/XMLSchema#string':
        return 'StringField'
    elif field_type == 'http://www.w3.org/2001/XMLSchema#boolean':
        return 'BooleanField'
    elif field_type == 'http://www.w3.org/2001/XMLSchema#integer':
        return 'IntegerField'
    elif field_type == 'http://www.w3.org/2001/XMLSchema#nonNegativeInteger':
        return 'NonNegIntegerField'
    elif field_type == 'http://www.w3.org/2001/XMLSchema#date':
        return 'DateField'
    elif field_type == 'http://www.w3.org/2001/XMLSchema#dateTime':
        return 'DateTimeField'
    elif field_type == 'http://www.w3.org/2001/XMLSchema#time':
        return 'TimeField'
    elif field_type == 'http://www.w3.org/2001/XMLSchema#anyURI':
        return 'URIField'
    elif field_type == 'https://schema.org/OpeningHoursSpecification':
        return 'DtcOpeningsurenSpecificatie'
    elif field_type == 'https://schema.org/ContactPoint':
        return 'DtcContactinfo'
    elif field_type == 'http://www.w3.org/2000/01/rdf-schema#Literal':
        return 'StringField'
    else:
        raise NotImplemented('not supported field_type in get_single_field_from_type_uri()')


def get_non_single_field_from_type_uri(field_type: str, valid_uri_and_types: Dict):
    if '#Dtc' in field_type:
        type_name = field_type[field_type.find("#") + 1::]
        return ['ComplexField', type_name]
    if field_type in valid_uri_and_types.keys():
        return ['ComplexField', valid_uri_and_types[field_type]]
    if '#Dte' in field_type or '#KwantWrd' in field_type:
        type_name = field_type[field_type.find("#") + 1::]
        return ['ComplexField', type_name]
    if '#Kl' in field_type or field_type in [
            'https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#AntiParkeerpaalType',
            'https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#KLLuidsprekerVormgeving']:
        type_name = field_type[field_type.find("#") + 1::]
        return ['KeuzelijstField', type_name]
    if '#Dtu' in field_type:
        type_name = field_type[field_type.find("#") + 1::]
        return ['UnionTypeField', type_name]

    raise NotImplementedError(f'not supported field_type {field_type} in get_non_single_field_from_type_uri()')


def get_field_name_from_type_uri(attribuut_type, valid_uri_and_types):
    if attribuut_type.startswith('http://www.w3.org/2001/XMLSchema#'):
        return get_single_field_from_type_uri(attribuut_type)
    if attribuut_type.startswith("https://schema.org/"):
        return get_non_single_field_from_type_uri(attribuut_type, valid_uri_and_types)[1]
    return get_non_single_field_from_type_uri(attribuut_type, valid_uri_and_types)[1]


def get_attributen_by_type_field(oslo_collector, type_field, oslo_datatype):
    if type_field == 'Complex':
        return oslo_collector.find_complex_datatype_attributes_by_class_uri(oslo_datatype.objectUri)
    elif type_field == 'UnionType':
        return oslo_collector.find_union_datatype_attributes_by_class_uri(oslo_datatype.objectUri)
    else:
        return oslo_collector.find_primitive_datatype_attributes_by_class_uri(oslo_datatype.objectUri)


def get_type_name_of_complex_attribuut(type_uri: str, valid_uri_and_types):
    if '/ns/' in type_uri or type_uri.startswith('http://www.w3.org/2001/XMLSchema#'):
        return type_uri[type_uri.find('#') + 1::]
    if type_uri in valid_uri_and_types.keys():
        return valid_uri_and_types[type_uri]

    raise NotImplementedError(f"get_type_name_of_complex_attribuut fails to get typename from {type_uri}")


def get_type_link_from_attribuut(oslo_collector, attribuut):
    type_link = oslo_collector.find_type_link_by_uri(attribuut.type)
    if type_link is not None:
        return type_link


def get_fields_and_names_from_list_of_attributes(attributen, valid_uri_and_types):
    if len(attributen) == 0:
        return []

    primitive_types_list = list(
        filter(lambda t: t.type.startswith('http://www.w3.org/2001/XMLSchema#'), attributen))
    other_types_list = list(
        filter(lambda t: not t.type.startswith('http://www.w3.org/2001/XMLSchema#'), attributen))

    select_types_list = list(
        map(lambda a: (get_single_field_from_type_uri(a.type), a.name), primitive_types_list))

    for nonPrimitiveType in other_types_list:
        select_types_list.append((get_field_name_from_type_uri(nonPrimitiveType.type, valid_uri_and_types), nonPrimitiveType.name))

    distinct_types_list = list(set(select_types_list))
    sorted_list = sorted(distinct_types_list, key=lambda t: t)
    return sorted_list


def get_type_name_of_union_attribuut(type_uri: str):
    if '/ns/' in type_uri and '#' in type_uri:
        return type_uri[type_uri.find('#') + 1::]

    raise NotImplementedError(f'get_type_name_of_complex_attribuut fails to get typename from {type_uri}')
